@@ -3,11 +3,23 @@ function(x, k, m = 1, control = list())
 {
     ## Partition a cluster ensemble x into (at most) k classes by
     ## minimizing
-    ##   \sum_b \sum_j u_{bj}^m d(x_b, p_j)
+    ##   \sum_b \sum_j u_{bj}^m d(x_b, p_j) ^ 2
     ## for suitable soft partition prototypes p_1, ..., p_k, where
     ## 1 <= m < \infty, with 1 corresponding to hard (secondary)
     ## partitions, and d is euclidean dissimilarity.
 
+    ## <NOTE>
+    ## The algorithm actually more generally works for minimizing
+    ##   \sum_b \sum_j u_{bj}^m d(x_b, p_j)
+    ## for "suitable" dissimilarity measures d (such as squared
+    ## Euclidean dissimilarity in the above), provided that suitable
+    ## consensus methods for solving
+    ##   \sum_b u_{bj}^m d(x_b, p) => \min_p
+    ## are available.
+    ## For the time being, we only work with squared Euclidean
+    ## dissimilarities.
+    ## </NOTE>
+
     ## Control parameters.
     maxiter <- control$maxiter
     if(is.null(maxiter))
@@ -15,7 +27,7 @@ function(x, k, m = 1, control = list())
     reltol <- control$reltol
     if(is.null(reltol))
         reltol <- sqrt(.Machine$double.eps)
-    ## To be passed on to cl_median().
+    ## To be passed on to cl_consensus().
     method <- control$method
     ## Do this at last ...
     control <- as.list(control$control)
@@ -32,7 +44,7 @@ function(x, k, m = 1, control = list())
     ## It may be better to use random soft partitions.
     prototypes <-
         memberships[sample(1 : B, k)]
-    dissimilarities <- cl_dissimilarity(memberships, prototypes)
+    dissimilarities <- cl_dissimilarity(memberships, prototypes) ^ 2
 
     if(m == 1) {
         ## Hard secondary partitions.
@@ -45,15 +57,16 @@ function(x, k, m = 1, control = list())
             ## <NOTE>
             ## Splitting lists is broken in R versions up to 2.0.1, so
             ## we use a loop here.  Something based on
-            ##   lapply(split(memberships, class_ids), cl_median)
+            ##   lapply(split(memberships, class_ids), cl_consensus)
             ## would be nicer ...
             for(j in unique(class_ids))
                 prototypes[[j]] <-
-                    cl_median(memberships[class_ids %in% j],
-                              method = method, control = control)
+                    cl_consensus(memberships[class_ids %in% j],
+                                 method = method, control = control)
             ## </NOTE>
             ## Update the class ids.
-            dissimilarities <- cl_dissimilarity(memberships, prototypes)
+            dissimilarities <-
+                cl_dissimilarity(memberships, prototypes) ^ 2
             class_ids <- max.col( - dissimilarities )
             new_value <-
                 sum(.one_entry_per_column(dissimilarities, class_ids))
@@ -94,14 +107,15 @@ function(x, k, m = 1, control = list())
             ## Update the prototypes.
             ## This amounts to solving, for each j:
             ##   \sum_b u_{bj}^m d(x_b, p_j) => \min_p
-            ## I.e., p_j is the *weighted* median of the x_b with
-            ## corresponding weights u_{bj}^m.
+            ## I.e., p_j is the *weighted* least squares consensus
+            ## clustering of the x_b with corresponding weights u_{bj}^m.
             for(j in 1 : k)
                 prototypes[[j]] <-
-                    cl_median(memberships, weights = u[, j] ^ m,
-                              method = method, control = control)
+                    cl_consensus(memberships, weights = u[, j] ^ m,
+                                 method = method, control = control)
             ## Update u.
-            dissimilarities <- cl_dissimilarity(memberships, prototypes)
+            dissimilarities <-
+                cl_dissimilarity(memberships, prototypes) ^ 2
             u <- u_from_d(dissimilarities)
             new_value <- value(u, dissimilarities)
             if(abs(old_value - new_value)
@@ -113,7 +127,7 @@ function(x, k, m = 1, control = list())
         class_ids <- max.col(u)
     }
 
-    dissimilarities <- as.matrix(cl_dissimilarity(memberships))
+    dissimilarities <- as.matrix(cl_dissimilarity(memberships) ^ 2)
     ## Note that our dissimilarities inherit from "cl_proximity" but not
     ## "dist", and as.dist() is not a generic function.
     u <- cl_membership(as.cl_membership(u), k)
